@@ -377,8 +377,6 @@ public class GenericsType extends ASTNode {
             for (int i = 0; redirectBoundGenericTypes!=null && i < redirectBoundGenericTypes.length && match; i++) {
                 GenericsType redirectBoundType = redirectBoundGenericTypes[i];
                 GenericsType classNodeType = cnTypes[i];
-                // The following code has been commented out because it causes GROOVY-5415
-                // However, commenting doesn't make any test fail, which is curious...
                 if (classNodeType.isPlaceholder()) {
                     String name = classNodeType.getName();
                     if (redirectBoundType.isPlaceholder()) {
@@ -441,17 +439,15 @@ public class GenericsType extends ASTNode {
                                                         gt = classNodePlaceholders.get(gt.getName());
                                                     }
                                                 }
-                                                match = match &&
-                                                        (implementsInterfaceOrIsSubclassOf(classNodeType.getType(), gt.getType())
-                                                         || classNodeType.isCompatibleWith(gt.getType())); // workaround for GROOVY-6095
+                                                match = implementsInterfaceOrIsSubclassOf(classNodeType.getType(), gt.getType())
+                                                         || classNodeType.isCompatibleWith(gt.getType()); // workaround for GROOVY-6095
                                                 if (!match) break;
                                             }
                                         }
                                         return match;
-                                    } else {
+                                    } else if (classNodePlaceholders.containsKey(name)) {
                                         redirectBoundType = classNodePlaceholders.get(name);
                                     }
-
                                 }
                             }
                             match = redirectBoundType.isCompatibleWith(classNodeType.getType());
@@ -462,8 +458,7 @@ public class GenericsType extends ASTNode {
                     }
                 }
             }
-            if (!match) return false;
-            return true;
+            return match;
         }
     }
 
