@@ -62,7 +62,7 @@ public class CompilerConfiguration {
     private static final String[] ALLOWED_JDKS = { JDK4, JDK5, JDK6, JDK7, JDK8 };
 
     // Just call getVMVersion() once.
-    public static final String currentJVMVersion = getVMVersion();
+    public static final String CURRENT_JVM_VERSION = getVMVersion();
 
     // Static initializers are executed in text order,
     // therefore we must do this one last!
@@ -666,7 +666,7 @@ public class CompilerConfiguration {
 
     public ParserPluginFactory getPluginFactory() {
         if (pluginFactory == null) {
-            pluginFactory = ParserPluginFactory.newInstance(true);
+            pluginFactory = ParserPluginFactory.newInstance();
         }
         return pluginFactory;
     }
