@@ -27,16 +27,28 @@ import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerFactory;
 import org.gradle.api.internal.initialization.ScriptHandlerInternal;
 import org.gradle.api.internal.plugins.PluginManagerInternal;
+import org.gradle.plugin.repository.internal.PluginRepositoryFactory;
+import org.gradle.plugin.repository.internal.PluginRepositoryRegistry;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.tasks.util.PatternSet;
 import org.gradle.api.tasks.util.internal.PatternSets;
-import org.gradle.groovy.scripts.*;
-import org.gradle.groovy.scripts.internal.*;
+import org.gradle.groovy.scripts.BasicScript;
+import org.gradle.groovy.scripts.ScriptCompiler;
+import org.gradle.groovy.scripts.ScriptCompilerFactory;
+import org.gradle.groovy.scripts.ScriptRunner;
+import org.gradle.groovy.scripts.ScriptSource;
+import org.gradle.groovy.scripts.internal.BuildScriptData;
+import org.gradle.groovy.scripts.internal.BuildScriptDataSerializer;
+import org.gradle.groovy.scripts.internal.BuildScriptTransformer;
+import org.gradle.groovy.scripts.internal.CompileOperation;
+import org.gradle.groovy.scripts.internal.FactoryBackedCompileOperation;
+import org.gradle.groovy.scripts.internal.InitialPassStatementTransformer;
+import org.gradle.groovy.scripts.internal.SubsetScriptTransformer;
 import org.gradle.internal.Actions;
 import org.gradle.internal.Factory;
+import org.gradle.internal.logging.LoggingManagerInternal;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.DefaultServiceRegistry;
-import org.gradle.logging.LoggingManagerInternal;
 import org.gradle.model.dsl.internal.transform.ClosureCreationInterceptingVerifier;
 import org.gradle.model.internal.inspect.ModelRuleSourceDetector;
 import org.gradle.plugin.use.internal.PluginRequestApplicator;
@@ -57,6 +69,8 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
     private final ModelRuleSourceDetector modelRuleSourceDetector;
     private final BuildScriptDataSerializer buildScriptDataSerializer = new BuildScriptDataSerializer();
     private final PluginRequestsSerializer pluginRequestsSerializer = new PluginRequestsSerializer();
+    private final PluginRepositoryRegistry pluginRepositoryRegistry;
+    private final PluginRepositoryFactory pluginRepositoryFactory;
 
     public DefaultScriptPluginFactory(ScriptCompilerFactory scriptCompilerFactory,
                                       Factory<LoggingManagerInternal> loggingManagerFactory,
@@ -66,7 +80,9 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
                                       FileLookup fileLookup,
                                       DirectoryFileTreeFactory directoryFileTreeFactory,
                                       DocumentationRegistry documentationRegistry,
-                                      ModelRuleSourceDetector modelRuleSourceDetector) {
+                                      ModelRuleSourceDetector modelRuleSourceDetector,
+                                      PluginRepositoryRegistry pluginRepositoryRegistry,
+                                      PluginRepositoryFactory pluginRepositoryFactory) {
         this.scriptCompilerFactory = scriptCompilerFactory;
         this.loggingManagerFactory = loggingManagerFactory;
         this.instantiator = instantiator;
@@ -76,6 +92,8 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
         this.directoryFileTreeFactory = directoryFileTreeFactory;
         this.documentationRegistry = documentationRegistry;
         this.modelRuleSourceDetector = modelRuleSourceDetector;
+        this.pluginRepositoryRegistry = pluginRepositoryRegistry;
+        this.pluginRepositoryFactory = pluginRepositoryFactory;
     }
 
     public ScriptPlugin create(ScriptSource scriptSource, ScriptHandler scriptHandler, ClassLoaderScope targetScope, ClassLoaderScope baseScope, boolean topLevelScript) {
@@ -116,31 +134,33 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
             services.add(FileLookup.class, fileLookup);
             services.add(DirectoryFileTreeFactory.class, directoryFileTreeFactory);
             services.add(ModelRuleSourceDetector.class, modelRuleSourceDetector);
+            services.add(PluginRepositoryRegistry.class, pluginRepositoryRegistry);
+            services.add(PluginRepositoryFactory.class, pluginRepositoryFactory);
 
-            final ScriptTarget scriptTarget = wrap(target);
+            final ScriptTarget initialPassScriptTarget = initialPassTarget(target);
 
             ScriptCompiler compiler = scriptCompilerFactory.createCompiler(scriptSource);
 
-            // Pass 1, extract plugin requests and execute buildscript {}, ignoring (i.e. not even compiling) anything else
+            // Pass 1, extract plugin requests and plugin repositories and execute buildscript {}, ignoring (i.e. not even compiling) anything else
 
-            Class<? extends BasicScript> scriptType = scriptTarget.getScriptClass();
-            boolean supportsPluginsBlock = scriptTarget.getSupportsPluginsBlock();
-            String onPluginBlockError = supportsPluginsBlock ? null : "Only Project build scripts can contain plugins {} blocks";
-            String classpathClosureName = scriptTarget.getClasspathBlockName();
-            InitialPassStatementTransformer initialPassStatementTransformer = new InitialPassStatementTransformer(classpathClosureName, onPluginBlockError, scriptSource, documentationRegistry);
+            Class<? extends BasicScript> scriptType = initialPassScriptTarget.getScriptClass();
+            InitialPassStatementTransformer initialPassStatementTransformer = new InitialPassStatementTransformer(scriptSource, initialPassScriptTarget, documentationRegistry);
             SubsetScriptTransformer initialTransformer = new SubsetScriptTransformer(initialPassStatementTransformer);
-            String id = INTERNER.intern("cp_" + scriptTarget.getId());
+            String id = INTERNER.intern("cp_" + initialPassScriptTarget.getId());
             CompileOperation<PluginRequests> initialOperation = new FactoryBackedCompileOperation<PluginRequests>(id, initialTransformer, initialPassStatementTransformer, pluginRequestsSerializer);
 
             ScriptRunner<? extends BasicScript, PluginRequests> initialRunner = compiler.compile(scriptType, initialOperation, baseScope.getExportClassLoader(), Actions.doNothing());
             initialRunner.run(target, services);
 
             PluginRequests pluginRequests = initialRunner.getData();
-            PluginManagerInternal pluginManager = scriptTarget.getPluginManager();
+            PluginManagerInternal pluginManager = initialPassScriptTarget.getPluginManager();
             pluginRequestApplicator.applyPlugins(pluginRequests, scriptHandler, pluginManager, targetScope);
 
-            // Pass 2, compile everything except buildscript {} and plugin requests, then run
-            BuildScriptTransformer buildScriptTransformer = new BuildScriptTransformer(classpathClosureName, scriptSource);
+            // Pass 2, compile everything except buildscript {}, pluginRepositories{}, and plugin requests, then run
+            final ScriptTarget scriptTarget = secondPassTarget(target);
+            scriptType = scriptTarget.getScriptClass();
+
+            BuildScriptTransformer buildScriptTransformer = new BuildScriptTransformer(scriptSource, scriptTarget);
             String operationId = scriptTarget.getId();
             CompileOperation<BuildScriptData> operation = new FactoryBackedCompileOperation<BuildScriptData>(operationId, buildScriptTransformer, buildScriptTransformer, buildScriptDataSerializer);
 
@@ -162,7 +182,15 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
             scriptTarget.addConfiguration(buildScriptRunner, !hasImperativeStatements);
         }
 
-        private ScriptTarget wrap(Object target) {
+        private ScriptTarget initialPassTarget(Object target) {
+            return wrap(target, true /* isInitialPass */);
+        }
+
+        private ScriptTarget secondPassTarget(Object target) {
+            return wrap(target, false /* isInitialPass */);
+        }
+
+        private ScriptTarget wrap(Object target, boolean isInitialPass) {
             if (target instanceof ProjectInternal && topLevelScript) {
                 // Only use this for top level project scripts
                 return new ProjectScriptTarget((ProjectInternal) target);
@@ -173,12 +201,14 @@ public class DefaultScriptPluginFactory implements ScriptPluginFactory {
             }
             if (target instanceof SettingsInternal && topLevelScript) {
                 // Only use this for top level settings scripts
-                return new SettingScriptTarget((SettingsInternal) target);
+                if (isInitialPass) {
+                    return new InitialPassSettingScriptTarget((SettingsInternal) target);
+                } else {
+                    return new SettingScriptTarget((SettingsInternal) target);
+                }
             } else {
                 return new DefaultScriptTarget(target);
             }
         }
-
     }
-
 }
