@@ -17,21 +17,17 @@ package org.gradle.initialization;
 
 import org.gradle.BuildListener;
 import org.gradle.BuildResult;
-import org.gradle.api.Transformer;
 import org.gradle.api.internal.ExceptionAnalyser;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.configuration.BuildConfigurer;
-import org.gradle.execution.BuildConfigurationAction;
 import org.gradle.execution.BuildConfigurationActionExecuter;
 import org.gradle.execution.BuildExecuter;
 import org.gradle.internal.Factory;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.progress.BuildOperationExecutor;
-import org.gradle.logging.LoggingManagerInternal;
-
-import java.io.Closeable;
-import java.util.List;
+import org.gradle.internal.service.scopes.BuildScopeServices;
+import org.gradle.internal.logging.LoggingManagerInternal;
 
 public class DefaultGradleLauncher extends GradleLauncher {
 
@@ -49,9 +45,9 @@ public class DefaultGradleLauncher extends GradleLauncher {
     private final ModelConfigurationListener modelConfigurationListener;
     private final BuildCompletionListener buildCompletionListener;
     private final BuildOperationExecutor buildOperationExecutor;
-    private BuildConfigurationActionExecuter buildConfigurationActionExecuter;
-    private BuildExecuter buildExecuter;
-    private final Closeable buildServices;
+    private final BuildConfigurationActionExecuter buildConfigurationActionExecuter;
+    private final BuildExecuter buildExecuter;
+    private final BuildScopeServices buildServices;
 
     /**
      * Creates a new instance.
@@ -61,7 +57,7 @@ public class DefaultGradleLauncher extends GradleLauncher {
                                  LoggingManagerInternal loggingManager, BuildListener buildListener,
                                  ModelConfigurationListener modelConfigurationListener,
                                  BuildCompletionListener buildCompletionListener, BuildOperationExecutor operationExecutor,
-                                 BuildConfigurationActionExecuter buildConfigurationActionExecuter, BuildExecuter buildExecuter, Closeable buildServices) {
+                                 BuildConfigurationActionExecuter buildConfigurationActionExecuter, BuildExecuter buildExecuter, BuildScopeServices buildServices) {
         this.gradle = gradle;
         this.initScriptHandler = initScriptHandler;
         this.settingsLoader = settingsLoader;
@@ -75,6 +71,7 @@ public class DefaultGradleLauncher extends GradleLauncher {
         this.buildExecuter = buildExecuter;
         this.buildCompletionListener = buildCompletionListener;
         this.buildServices = buildServices;
+        loggingManager.start();
     }
 
     public GradleInternal getGradle() {
@@ -92,8 +89,6 @@ public class DefaultGradleLauncher extends GradleLauncher {
     }
 
     private BuildResult doBuild(final Stage upTo) {
-        loggingManager.start();
-
         return buildOperationExecutor.run("Run build", new Factory<BuildResult>() {
             @Override
             public BuildResult create() {
@@ -104,7 +99,7 @@ public class DefaultGradleLauncher extends GradleLauncher {
                 } catch (Throwable t) {
                     failure = exceptionAnalyser.transform(t);
                 }
-                BuildResult buildResult = new BuildResult(gradle, failure);
+                BuildResult buildResult = new BuildResult(upTo.name(), gradle, failure);
                 buildListener.buildFinished(buildResult);
                 if (failure != null) {
                     throw new ReportedException(failure);
@@ -163,9 +158,8 @@ public class DefaultGradleLauncher extends GradleLauncher {
     }
 
     /**
-     * <p>Adds a listener to this build instance. The listener is notified of events which occur during the
-     * execution of the build. See {@link org.gradle.api.invocation.Gradle#addListener(Object)} for supported listener
-     * types.</p>
+     * <p>Adds a listener to this build instance. The listener is notified of events which occur during the execution of the build. See {@link org.gradle.api.invocation.Gradle#addListener(Object)} for
+     * supported listener types.</p>
      *
      * @param listener The listener to add. Has no effect if the listener has already been added.
      */
@@ -175,8 +169,7 @@ public class DefaultGradleLauncher extends GradleLauncher {
     }
 
     /**
-     * <p>Adds a {@link StandardOutputListener} to this build instance. The listener is notified of any text written to
-     * standard output by Gradle's logging system
+     * <p>Adds a {@link StandardOutputListener} to this build instance. The listener is notified of any text written to standard output by Gradle's logging system
      *
      * @param listener The listener to add. Has no effect if the listener has already been added.
      */
@@ -186,8 +179,7 @@ public class DefaultGradleLauncher extends GradleLauncher {
     }
 
     /**
-     * <p>Adds a {@link StandardOutputListener} to this build instance. The listener is notified of any text written to
-     * standard error by Gradle's logging system
+     * <p>Adds a {@link StandardOutputListener} to this build instance. The listener is notified of any text written to standard error by Gradle's logging system
      *
      * @param listener The listener to add. Has no effect if the listener has already been added.
      */
@@ -196,10 +188,6 @@ public class DefaultGradleLauncher extends GradleLauncher {
         loggingManager.addStandardErrorListener(listener);
     }
 
-    public void registerBuildConfigurationTransformer(Transformer<List<BuildConfigurationAction>, List<BuildConfigurationAction>> transformer) {
-        buildConfigurationActionExecuter.registerBuildConfigurationTransformer(transformer);
-    }
-
     public void stop() {
         try {
             loggingManager.stop();
