@@ -17,8 +17,10 @@
 package org.gradle.internal.filewatch;
 
 import org.gradle.api.internal.file.FileSystemSubset;
-import org.gradle.initialization.BuildCancellationToken;
+import org.gradle.internal.concurrent.Stoppable;
 
-public interface FileSystemChangeWaiter {
-    void wait(FileSystemSubset taskFileSystemInputs, BuildCancellationToken cancellationToken, Runnable notifier);
+public interface FileSystemChangeWaiter extends Stoppable {
+    void watch(FileSystemSubset fileSystemSubset);
+    void wait(Runnable notifier, FileWatcherEventListener eventListener);
+    boolean isWatching();
 }
