@@ -16,7 +16,7 @@
 package org.gradle.tooling.internal.adapter;
 
 import com.google.common.base.Optional;
-import org.gradle.api.Action;
+import org.gradle.api.Nullable;
 import org.gradle.internal.UncheckedException;
 import org.gradle.internal.reflect.DirectInstantiator;
 import org.gradle.internal.typeconversion.EnumFromCharSequenceNotationParser;
@@ -30,31 +30,44 @@ import org.gradle.tooling.model.internal.ImmutableDomainObjectSet;
 import java.io.IOException;
 import java.io.Serializable;
 import java.lang.ref.SoftReference;
-import java.lang.reflect.*;
-import java.util.*;
+import java.lang.reflect.InvocationHandler;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.lang.reflect.ParameterizedType;
+import java.lang.reflect.Proxy;
+import java.lang.reflect.Type;
+import java.lang.reflect.WildcardType;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 /**
  * Adapts some source object to some target view type.
  */
-public class ProtocolToModelAdapter implements Serializable {
-    private static final MethodInvoker NO_OP_HANDLER = new NoOpMethodInvoker();
-    private static final Action<SourceObjectMapping> NO_OP_MAPPER = new NoOpMapping();
+public class ProtocolToModelAdapter implements ObjectGraphAdapter {
+    private static final ViewDecoration NO_OP_MAPPER = new NoOpDecoration();
     private static final TargetTypeProvider IDENTITY_TYPE_PROVIDER = new TargetTypeProvider() {
         public <T> Class<? extends T> getTargetType(Class<T> initialTargetType, Object protocolObject) {
             return initialTargetType;
         }
     };
+    private static final ReflectionMethodInvoker REFLECTION_METHOD_INVOKER = new ReflectionMethodInvoker();
+    private static final TypeInspector TYPE_INSPECTOR = new TypeInspector();
+    private static final CollectionMapper COLLECTION_MAPPER = new CollectionMapper();
     private static final Object[] EMPTY = new Object[0];
-    private static final Pattern IS_SUPPORT_METHOD = Pattern.compile("is(\\w+)Supported");
-    private final TargetTypeProvider targetTypeProvider;
-    private final CollectionMapper collectionMapper = new CollectionMapper();
-
+    private static final Class[] EMPTY_CLASS_ARRAY = new Class[0];
     private static final Method EQUALS_METHOD;
     private static final Method HASHCODE_METHOD;
 
+    private final TargetTypeProvider targetTypeProvider;
+
     static {
         Method equalsMethod;
         Method hashCodeMethod;
@@ -77,74 +90,72 @@ public class ProtocolToModelAdapter implements Serializable {
     }
 
     /**
-     * Adapts the source object to a view object.
+     * Creates an adapter for a single object graph. Each object adapted by the returned adapter is treated as part of the same object graph, for the purposes of caching etc.
      */
-    public <T, S> T adapt(Class<T> targetType, S sourceObject) {
-        return adapt(targetType, sourceObject, NO_OP_MAPPER);
-    }
-
-    /**
-     * Adapts the source object to a view object.
-     *
-     * @param mixInClass A bean that provides implementations for methods of the target type. If this bean implements the given method, it is preferred over the source object's implementation.
-     */
-    public <T, S> T adapt(Class<T> targetType, final S sourceObject, final Class<?> mixInClass) {
-        return adapt(targetType, sourceObject, new Action<SourceObjectMapping>() {
-            public void execute(SourceObjectMapping mapping) {
-                if (mapping.getSourceObject() == sourceObject) {
-                    mapping.mixIn(mixInClass);
-                }
+    public ObjectGraphAdapter newGraph() {
+        final ViewGraphDetails graphDetails = new ViewGraphDetails(targetTypeProvider);
+        return new ObjectGraphAdapter() {
+            @Override
+            public <T> T adapt(Class<T> targetType, Object sourceObject) {
+                return createView(targetType, sourceObject, NO_OP_MAPPER, graphDetails);
             }
-        });
+
+            @Override
+            public <T> ViewBuilder<T> builder(Class<T> viewType) {
+                return new DefaultViewBuilder<T>(viewType, graphDetails);
+            }
+        };
     }
 
     /**
      * Adapts the source object to a view object.
-     *
-     * @param mapper An action that is invoked for each source object in the graph that is to be adapted. The action can influence how the source object is adapted via the provided {@link
-     * SourceObjectMapping}.
      */
-    /**
-     * Adapts the source object to a view object.
-     *
-     * @param mapper An action that is invoked for each source object in the graph that is to be adapted. The action can influence how the source object is adapted via the provided {@link
-     * SourceObjectMapping}.
-     */
-    public <T, S> T adapt(Class<T> targetType, S sourceObject, Action<? super SourceObjectMapping> mapper) {
+    @Override
+    public <T> T adapt(Class<T> targetType, Object sourceObject) {
         if (sourceObject == null) {
             return null;
         }
-        Class<? extends T> wrapperType = targetTypeProvider.getTargetType(targetType, sourceObject);
-        DefaultSourceObjectMapping mapping = new DefaultSourceObjectMapping(sourceObject, targetType, wrapperType);
-        mapper.execute(mapping);
-        wrapperType = mapping.wrapperType.asSubclass(targetType);
-        if (wrapperType.isInstance(sourceObject)) {
-            return wrapperType.cast(sourceObject);
+        return createView(targetType, sourceObject, NO_OP_MAPPER, new ViewGraphDetails(targetTypeProvider));
+    }
+
+    /**
+     * Creates a builder for views of the given type.
+     */
+    @Override
+    public <T> ViewBuilder<T> builder(final Class<T> viewType) {
+        return new DefaultViewBuilder<T>(viewType);
+    }
+
+    private static <T> T createView(Class<T> targetType, Object sourceObject, ViewDecoration decoration, ViewGraphDetails graphDetails) {
+        if (sourceObject == null) {
+            return null;
+        }
+
+        // Calculate the actual type
+        Class<? extends T> viewType = graphDetails.typeProvider.getTargetType(targetType, sourceObject);
+
+        if (viewType.isInstance(sourceObject)) {
+            return viewType.cast(sourceObject);
         }
         if (targetType.isEnum()) {
             return adaptToEnum(targetType, sourceObject);
         }
 
-        MixInMethodInvoker mixInMethodInvoker = null;
-        if (mapping.mixInType != null) {
-            mixInMethodInvoker = new MixInMethodInvoker(mapping.mixInType, new AdaptingMethodInvoker(mapper, new ReflectionMethodInvoker()));
-        }
-        MethodInvoker overrideInvoker = chainInvokers(mixInMethodInvoker, mapping.overrideInvoker);
-        Object proxy = Proxy.newProxyInstance(wrapperType.getClassLoader(), new Class<?>[]{wrapperType}, new InvocationHandlerImpl(sourceObject, overrideInvoker, mapper));
-        if (mixInMethodInvoker != null) {
-            mixInMethodInvoker.setProxy(proxy);
-        }
-        return wrapperType.cast(proxy);
-    }
+        // Restrict the decorations to those required to decorate all views reachable from this type
+        ViewDecoration decorationsForThisType = decoration.isNoOp() ? decoration : decoration.restrictTo(TYPE_INSPECTOR.getReachableTypes(targetType));
 
-    private MethodInvoker chainInvokers(MixInMethodInvoker mixInMethodInvoker, MethodInvoker overrideInvoker) {
-        if (mixInMethodInvoker == null) {
-            return overrideInvoker;
+        ViewKey viewKey = new ViewKey(viewType, sourceObject, decorationsForThisType);
+        Object view = graphDetails.views.get(viewKey);
+        if (view != null) {
+            return targetType.cast(view);
         }
-        if (overrideInvoker == NO_OP_HANDLER) {
-            return mixInMethodInvoker;
-        }
-        return new ChainedMethodInvoker(mixInMethodInvoker, overrideInvoker);
+
+        // Create a proxy
+        InvocationHandlerImpl handler = new InvocationHandlerImpl(targetType, sourceObject, decorationsForThisType, graphDetails);
+        Object proxy = Proxy.newProxyInstance(viewType.getClassLoader(), new Class<?>[]{viewType}, handler);
+        handler.attachProxy(proxy);
+
+        return viewType.cast(proxy);
     }
 
     private static <T, S> T adaptToEnum(Class<T> targetType, S sourceObject) {
@@ -165,19 +176,19 @@ public class ProtocolToModelAdapter implements Serializable {
         }
     }
 
-    private Object convert(Type targetType, Object sourceObject, Action<? super SourceObjectMapping> mapping) {
+    private static Object convert(Type targetType, Object sourceObject, ViewDecoration decoration, ViewGraphDetails graphDetails) {
         if (targetType instanceof ParameterizedType) {
             ParameterizedType parameterizedTargetType = (ParameterizedType) targetType;
             if (parameterizedTargetType.getRawType() instanceof Class) {
                 Class<?> rawClass = (Class<?>) parameterizedTargetType.getRawType();
                 if (Iterable.class.isAssignableFrom(rawClass)) {
                     Type targetElementType = getElementType(parameterizedTargetType, 0);
-                    return convertCollectionInternal(rawClass, targetElementType, (Iterable<?>) sourceObject, mapping);
+                    return convertCollectionInternal(rawClass, targetElementType, (Iterable<?>) sourceObject, decoration, graphDetails);
                 }
                 if (Map.class.isAssignableFrom(rawClass)) {
                     Type targetKeyType = getElementType(parameterizedTargetType, 0);
                     Type targetValueType = getElementType(parameterizedTargetType, 1);
-                    return convertMap(rawClass, targetKeyType, targetValueType, (Map<?, ?>) sourceObject, mapping);
+                    return convertMap(rawClass, targetKeyType, targetValueType, (Map<?, ?>) sourceObject, decoration, graphDetails);
                 }
             }
         }
@@ -185,22 +196,22 @@ public class ProtocolToModelAdapter implements Serializable {
             if (((Class) targetType).isPrimitive()) {
                 return sourceObject;
             }
-            return adapt((Class) targetType, sourceObject, mapping);
+            return createView((Class) targetType, sourceObject, decoration, graphDetails);
         }
         throw new UnsupportedOperationException(String.format("Cannot convert object of %s to %s.", sourceObject.getClass(), targetType));
     }
 
-    private Map<Object, Object> convertMap(Class<?> mapClass, Type targetKeyType, Type targetValueType, Map<?, ?> sourceObject, Action<? super SourceObjectMapping> mapping) {
-        Map<Object, Object> convertedElements = collectionMapper.createEmptyMap(mapClass);
+    private static Map<Object, Object> convertMap(Class<?> mapClass, Type targetKeyType, Type targetValueType, Map<?, ?> sourceObject, ViewDecoration decoration, ViewGraphDetails graphDetails) {
+        Map<Object, Object> convertedElements = COLLECTION_MAPPER.createEmptyMap(mapClass);
         for (Map.Entry<?, ?> entry : sourceObject.entrySet()) {
-            convertedElements.put(convert(targetKeyType, entry.getKey(), mapping), convert(targetValueType, entry.getValue(), mapping));
+            convertedElements.put(convert(targetKeyType, entry.getKey(), decoration, graphDetails), convert(targetValueType, entry.getValue(), decoration, graphDetails));
         }
         return convertedElements;
     }
 
-    private Object convertCollectionInternal(Class<?> collectionClass, Type targetElementType, Iterable<?> sourceObject, Action<? super SourceObjectMapping> mapping) {
-        Collection<Object> convertedElements = collectionMapper.createEmptyCollection(collectionClass);
-        convertCollectionInternal(convertedElements, targetElementType, sourceObject, mapping);
+    private static Object convertCollectionInternal(Class<?> collectionClass, Type targetElementType, Iterable<?> sourceObject, ViewDecoration decoration, ViewGraphDetails graphDetails) {
+        Collection<Object> convertedElements = COLLECTION_MAPPER.createEmptyCollection(collectionClass);
+        convertCollectionInternal(convertedElements, targetElementType, sourceObject, decoration, graphDetails);
         if (collectionClass.equals(DomainObjectSet.class)) {
             return new ImmutableDomainObjectSet(convertedElements);
         } else {
@@ -208,13 +219,13 @@ public class ProtocolToModelAdapter implements Serializable {
         }
     }
 
-    private void convertCollectionInternal(Collection<Object> targetCollection, Type targetElementType, Iterable<?> sourceObject, Action<? super SourceObjectMapping> mapping) {
+    private static void convertCollectionInternal(Collection<Object> targetCollection, Type targetElementType, Iterable<?> sourceObject, ViewDecoration viewDecoration, ViewGraphDetails graphDetails) {
         for (Object element : sourceObject) {
-            targetCollection.add(convert(targetElementType, element, mapping));
+            targetCollection.add(convert(targetElementType, element, viewDecoration, graphDetails));
         }
     }
 
-    private Type getElementType(ParameterizedType type, int index) {
+    private static Type getElementType(ParameterizedType type, int index) {
         Type elementType = type.getActualTypeArguments()[index];
         if (elementType instanceof WildcardType) {
             WildcardType wildcardType = (WildcardType) elementType;
@@ -231,84 +242,82 @@ public class ProtocolToModelAdapter implements Serializable {
             throw new IllegalArgumentException("The given object is not a view object");
         }
         InvocationHandlerImpl handler = (InvocationHandlerImpl) Proxy.getInvocationHandler(viewObject);
-        return handler.delegate;
+        return handler.sourceObject;
     }
 
-    private static class DefaultSourceObjectMapping implements SourceObjectMapping {
-        private final Object protocolObject;
+    private static class ViewGraphDetails implements Serializable {
+        // Transient, don't serialize all the views that happen to have been visited, recreate them when visited via the deserialized view
+        private transient Map<ViewKey, Object> views = new HashMap<ViewKey, Object>();
+        private final TargetTypeProvider typeProvider;
+
+        ViewGraphDetails(TargetTypeProvider typeProvider) {
+            this.typeProvider = typeProvider;
+        }
+
+        private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
+            in.defaultReadObject();
+            views = new HashMap<ViewKey, Object>();
+        }
+    }
+
+    private static class ViewKey implements Serializable {
+        private final Class<?> type;
+        private final Object source;
+        private final ViewDecoration viewDecoration;
+
+        ViewKey(Class<?> type, Object source, ViewDecoration viewDecoration) {
+            this.type = type;
+            this.source = source;
+            this.viewDecoration = viewDecoration;
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            ViewKey other = (ViewKey) obj;
+            return other.source == source && other.type.equals(type) && other.viewDecoration.equals(viewDecoration);
+        }
+
+        @Override
+        public int hashCode() {
+            return type.hashCode() ^ System.identityHashCode(source) ^ viewDecoration.hashCode();
+        }
+    }
+
+    private static class InvocationHandlerImpl implements InvocationHandler, Serializable {
         private final Class<?> targetType;
-        private Class<?> wrapperType;
-        private Class<?> mixInType;
-        private MethodInvoker overrideInvoker = NO_OP_HANDLER;
-
-        public DefaultSourceObjectMapping(Object protocolObject, Class<?> targetType, Class<?> wrapperType) {
-            this.protocolObject = protocolObject;
-            this.targetType = targetType;
-            this.wrapperType = wrapperType;
-        }
-
-        public Object getSourceObject() {
-            return protocolObject;
-        }
-
-        public Class<?> getTargetType() {
-            return targetType;
-        }
-
-        public void mapToType(Class<?> type) {
-            if (!targetType.isAssignableFrom(type)) {
-                throw new IllegalArgumentException(String.format("requested wrapper type '%s' is not assignable to target type '%s'.", type.getSimpleName(), targetType.getSimpleName()));
-            }
-            wrapperType = type;
-        }
-
-        public void mixIn(Class<?> mixInBeanType) {
-            if (mixInType != null) {
-                throw new UnsupportedOperationException("Mixing in multiple beans is currently not supported.");
-            }
-            mixInType = mixInBeanType;
-        }
-
-        public void mixIn(MethodInvoker invoker) {
-            if (overrideInvoker != NO_OP_HANDLER) {
-                throw new UnsupportedOperationException("Mixing in multiple invokers is currently not supported.");
-            }
-            overrideInvoker = invoker;
-        }
-    }
-
-    private static class NoOpMapping implements Action<SourceObjectMapping>, Serializable {
-        public void execute(SourceObjectMapping mapping) {
-        }
-    }
-
-    private class InvocationHandlerImpl implements InvocationHandler, Serializable {
-        private final Object delegate;
-        private final MethodInvoker overrideMethodInvoker;
-        private final Action<? super SourceObjectMapping> mapper;
+        private final Object sourceObject;
+        private final ViewDecoration decoration;
+        private final ViewGraphDetails graphDetails;
+        private Object proxy;
+        // Recreate the invoker when deserialized, rather than serialize all its state
         private transient MethodInvoker invoker;
 
-        public InvocationHandlerImpl(Object delegate, MethodInvoker overrideMethodInvoker, Action<? super SourceObjectMapping> mapper) {
-            this.delegate = delegate;
-            this.overrideMethodInvoker = overrideMethodInvoker;
-            this.mapper = mapper;
+        InvocationHandlerImpl(Class<?> targetType, Object sourceObject, ViewDecoration decoration, ViewGraphDetails graphDetails) {
+            this.targetType = targetType;
+            this.sourceObject = sourceObject;
+            this.decoration = decoration;
+            this.graphDetails = graphDetails;
             setup();
         }
 
-        private void readObject(java.io.ObjectInputStream in)
-            throws IOException, ClassNotFoundException {
+        private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
             in.defaultReadObject();
             setup();
+            graphDetails.views.put(new ViewKey(targetType, sourceObject, decoration), proxy);
         }
 
         private void setup() {
+            List<MethodInvoker> invokers = new ArrayList<MethodInvoker>();
+            invokers.add(REFLECTION_METHOD_INVOKER);
+            decoration.collectInvokers(sourceObject, targetType, invokers);
+
+            MethodInvoker mixInMethodInvoker = invokers.size() == 1 ? invokers.get(0) : new ChainedMethodInvoker(invokers);
+
             invoker = new SupportedPropertyInvoker(
                 new SafeMethodInvoker(
                     new PropertyCachingMethodInvoker(
-                        new AdaptingMethodInvoker(mapper,
-                            new ChainedMethodInvoker(
-                                overrideMethodInvoker,
-                                new ReflectionMethodInvoker())))));
+                        new AdaptingMethodInvoker(decoration, graphDetails,
+                            mixInMethodInvoker))));
         }
 
         @Override
@@ -321,12 +330,12 @@ public class ProtocolToModelAdapter implements Serializable {
             }
 
             InvocationHandlerImpl other = (InvocationHandlerImpl) o;
-            return delegate.equals(other.delegate);
+            return sourceObject.equals(other.sourceObject);
         }
 
         @Override
         public int hashCode() {
-            return delegate.hashCode();
+            return sourceObject.hashCode();
         }
 
         public Object invoke(Object target, Method method, Object[] params) throws Throwable {
@@ -341,7 +350,7 @@ public class ProtocolToModelAdapter implements Serializable {
                 return hashCode();
             }
 
-            MethodInvocation invocation = new MethodInvocation(method.getName(), method.getReturnType(), method.getGenericReturnType(), method.getParameterTypes(), delegate, params);
+            MethodInvocation invocation = new MethodInvocation(method.getName(), method.getReturnType(), method.getGenericReturnType(), method.getParameterTypes(), target, targetType, sourceObject, params);
             invoker.invoke(invocation);
             if (!invocation.found()) {
                 String methodName = method.getDeclaringClass().getSimpleName() + "." + method.getName() + "()";
@@ -349,13 +358,18 @@ public class ProtocolToModelAdapter implements Serializable {
             }
             return invocation.getResult();
         }
+
+        void attachProxy(Object proxy) {
+            this.proxy = proxy;
+            graphDetails.views.put(new ViewKey(targetType, sourceObject, decoration), proxy);
+        }
     }
 
     private static class ChainedMethodInvoker implements MethodInvoker {
         private final MethodInvoker[] invokers;
 
-        private ChainedMethodInvoker(MethodInvoker... invokers) {
-            this.invokers = invokers;
+        private ChainedMethodInvoker(List<MethodInvoker> invokers) {
+            this.invokers = invokers.toArray(new MethodInvoker[0]);
         }
 
         public void invoke(MethodInvocation method) throws Throwable {
@@ -366,19 +380,21 @@ public class ProtocolToModelAdapter implements Serializable {
         }
     }
 
-    private class AdaptingMethodInvoker implements MethodInvoker {
-        private final Action<? super SourceObjectMapping> mapping;
+    private static class AdaptingMethodInvoker implements MethodInvoker {
+        private final ViewDecoration decoration;
+        private final ViewGraphDetails graphDetails;
         private final MethodInvoker next;
 
-        private AdaptingMethodInvoker(Action<? super SourceObjectMapping> mapping, MethodInvoker next) {
-            this.mapping = mapping;
+        private AdaptingMethodInvoker(ViewDecoration decoration, ViewGraphDetails graphDetails, MethodInvoker next) {
+            this.decoration = decoration;
+            this.graphDetails = graphDetails;
             this.next = next;
         }
 
         public void invoke(MethodInvocation invocation) throws Throwable {
             next.invoke(invocation);
             if (invocation.found() && invocation.getResult() != null) {
-                invocation.setResult(convert(invocation.getGenericReturnType(), invocation.getResult(), mapping));
+                invocation.setResult(convert(invocation.getGenericReturnType(), invocation.getResult(), decoration, graphDetails));
             }
         }
     }
@@ -562,7 +578,7 @@ public class ProtocolToModelAdapter implements Serializable {
     }
 
     private static class ReflectionMethodInvoker implements MethodInvoker {
-        private final static MethodInvocationCache LOOKUP_CACHE = new MethodInvocationCache();
+        private final MethodInvocationCache lookupCache = new MethodInvocationCache();
 
         public void invoke(MethodInvocation invocation) throws Throwable {
             Method targetMethod = locateMethod(invocation);
@@ -580,10 +596,9 @@ public class ProtocolToModelAdapter implements Serializable {
             invocation.setResult(returnValue);
         }
 
-        private static Method locateMethod(MethodInvocation invocation) {
-            return LOOKUP_CACHE.get(invocation);
+        private Method locateMethod(MethodInvocation invocation) {
+            return lookupCache.get(invocation);
         }
-
     }
 
     private static class PropertyCachingMethodInvoker implements MethodInvoker {
@@ -621,7 +636,6 @@ public class ProtocolToModelAdapter implements Serializable {
     }
 
     private static class SafeMethodInvoker implements MethodInvoker {
-        private static final Class[] EMPTY_CLASS_ARRAY = new Class[0];
         private final MethodInvoker next;
 
         private SafeMethodInvoker(MethodInvoker next) {
@@ -630,15 +644,11 @@ public class ProtocolToModelAdapter implements Serializable {
 
         public void invoke(MethodInvocation invocation) throws Throwable {
             next.invoke(invocation);
-            if (invocation.found() || invocation.getParameterTypes().length != 1) {
+            if (invocation.found() || invocation.getParameterTypes().length != 1 || !invocation.isIsOrGet()) {
                 return;
             }
 
-            if (!invocation.isIsOrGet()) {
-                return;
-            }
-
-            MethodInvocation getterInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), EMPTY_CLASS_ARRAY, invocation.getDelegate(), EMPTY);
+            MethodInvocation getterInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), EMPTY_CLASS_ARRAY, invocation.getView(), invocation.getViewType(), invocation.getDelegate(), EMPTY);
             next.invoke(getterInvocation);
             if (getterInvocation.found() && getterInvocation.getResult() != null) {
                 invocation.setResult(getterInvocation.getResult());
@@ -656,27 +666,60 @@ public class ProtocolToModelAdapter implements Serializable {
         }
 
         public void invoke(MethodInvocation invocation) throws Throwable {
-            Matcher matcher = IS_SUPPORT_METHOD.matcher(invocation.getName());
-            if (!matcher.matches()) {
-                next.invoke(invocation);
+            next.invoke(invocation);
+            if (invocation.found()) {
                 return;
             }
 
-            String getterName = String.format("get%s", matcher.group(1));
-            MethodInvocation getterInvocation = new MethodInvocation(getterName, invocation.getReturnType(), invocation.getGenericReturnType(), new Class[0], invocation.getDelegate(), EMPTY);
+            String methodName = invocation.getName();
+            boolean isSupportMethod = methodName.length() > 11 && methodName.startsWith("is") && methodName.endsWith("Supported");
+            if (!isSupportMethod) {
+                return;
+            }
+
+            String getterName = "get" + methodName.substring(2, methodName.length() - 9);
+            MethodInvocation getterInvocation = new MethodInvocation(getterName, invocation.getReturnType(), invocation.getGenericReturnType(), EMPTY_CLASS_ARRAY, invocation.getView(), invocation.getViewType(), invocation.getDelegate(), EMPTY);
             next.invoke(getterInvocation);
             invocation.setResult(getterInvocation.found());
         }
     }
 
-    private static class MixInMethodInvoker implements MethodInvoker {
-        private Object proxy;
+    private static class BeanMixInMethodInvoker implements MethodInvoker {
+        private final Object instance;
+        private final MethodInvoker next;
+
+        BeanMixInMethodInvoker(Object instance, MethodInvoker next) {
+            this.instance = instance;
+            this.next = next;
+        }
+
+        @Override
+        public void invoke(MethodInvocation invocation) throws Throwable {
+            MethodInvocation beanInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), invocation.getParameterTypes(), invocation.getView(), invocation.getViewType(), instance, invocation.getParameters());
+            next.invoke(beanInvocation);
+            if (beanInvocation.found()) {
+                invocation.setResult(beanInvocation.getResult());
+                return;
+            }
+            if (!invocation.isGetter()) {
+                return;
+            }
+
+            beanInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), new Class<?>[]{invocation.getViewType()}, invocation.getView(), invocation.getViewType(), instance, new Object[]{invocation.getView()});
+            next.invoke(beanInvocation);
+            if (beanInvocation.found()) {
+                invocation.setResult(beanInvocation.getResult());
+            }
+        }
+    }
+
+    private static class ClassMixInMethodInvoker implements MethodInvoker {
         private Object instance;
         private final Class<?> mixInClass;
         private final MethodInvoker next;
         private final ThreadLocal<MethodInvocation> current = new ThreadLocal<MethodInvocation>();
 
-        public MixInMethodInvoker(Class<?> mixInClass, MethodInvoker next) {
+        ClassMixInMethodInvoker(Class<?> mixInClass, MethodInvoker next) {
             this.mixInClass = mixInClass;
             this.next = next;
         }
@@ -688,9 +731,9 @@ public class ProtocolToModelAdapter implements Serializable {
             }
 
             if (instance == null) {
-                instance = DirectInstantiator.INSTANCE.newInstance(mixInClass, proxy);
+                instance = DirectInstantiator.INSTANCE.newInstance(mixInClass, invocation.getView());
             }
-            MethodInvocation beanInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), invocation.getParameterTypes(), instance, invocation.getParameters());
+            MethodInvocation beanInvocation = new MethodInvocation(invocation.getName(), invocation.getReturnType(), invocation.getGenericReturnType(), invocation.getParameterTypes(), invocation.getView(), invocation.getViewType(), instance, invocation.getParameters());
             current.set(beanInvocation);
             try {
                 next.invoke(beanInvocation);
@@ -701,13 +744,242 @@ public class ProtocolToModelAdapter implements Serializable {
                 invocation.setResult(beanInvocation.getResult());
             }
         }
+    }
 
-        public void setProxy(Object proxy) {
-            this.proxy = proxy;
+    private interface ViewDecoration {
+        void collectInvokers(Object sourceObject, Class<?> viewType, List<MethodInvoker> invokers);
+
+        boolean isNoOp();
+
+        /**
+         * Filter this decoration to apply only to the given view types. Return {@link #NO_OP_MAPPER} if this decoration does not apply to any of the types.
+         */
+        ViewDecoration restrictTo(Set<Class<?>> viewTypes);
+    }
+
+    private static class NoOpDecoration implements ViewDecoration, Serializable {
+        @Override
+        public void collectInvokers(Object sourceObject, Class<?> viewType, List<MethodInvoker> invokers) {
         }
 
-        public Object getProxy() {
-            return proxy;
+        @Override
+        public boolean equals(Object obj) {
+            return obj instanceof NoOpDecoration;
+        }
+
+        @Override
+        public int hashCode() {
+            return 0;
+        }
+
+        @Override
+        public boolean isNoOp() {
+            return true;
+        }
+
+        @Override
+        public ViewDecoration restrictTo(Set<Class<?>> viewTypes) {
+            return this;
+        }
+    }
+
+    private static class MixInMappingAction implements ViewDecoration, Serializable {
+        private final List<? extends ViewDecoration> decorations;
+
+        private MixInMappingAction(List<? extends ViewDecoration> decorations) {
+            assert decorations.size() >= 2;
+            this.decorations = decorations;
+        }
+
+        static ViewDecoration chain(List<? extends ViewDecoration> decorations) {
+            if (decorations.isEmpty()) {
+                return NO_OP_MAPPER;
+            }
+            if (decorations.size() == 1) {
+                return decorations.get(0);
+            }
+            return new MixInMappingAction(decorations);
+        }
+
+        @Override
+        public int hashCode() {
+            int v = 0;
+            for (ViewDecoration decoration : decorations) {
+                v = v ^ decoration.hashCode();
+            }
+            return v;
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            if (!obj.getClass().equals(MixInMappingAction.class)) {
+                return false;
+            }
+            MixInMappingAction other = (MixInMappingAction) obj;
+            return decorations.equals(other.decorations);
+        }
+
+        @Override
+        public boolean isNoOp() {
+            for (ViewDecoration decoration : decorations) {
+                if (!decoration.isNoOp()) {
+                    return false;
+                }
+            }
+            return true;
+        }
+
+        @Override
+        public ViewDecoration restrictTo(Set<Class<?>> viewTypes) {
+            List<ViewDecoration> filtered = new ArrayList<ViewDecoration>();
+            for (ViewDecoration viewDecoration : decorations) {
+                ViewDecoration filteredDecoration = viewDecoration.restrictTo(viewTypes);
+                if (!filteredDecoration.isNoOp()) {
+                    filtered.add(filteredDecoration);
+                }
+            }
+            if (filtered.size() == 0) {
+                return NO_OP_MAPPER;
+            }
+            if (filtered.size() == 1) {
+                return filtered.get(0);
+            }
+            if (filtered.equals(decorations)) {
+                return this;
+            }
+            return new MixInMappingAction(filtered);
+        }
+
+        @Override
+        public void collectInvokers(Object sourceObject, Class<?> viewType, List<MethodInvoker> invokers) {
+            for (ViewDecoration decoration : decorations) {
+                decoration.collectInvokers(sourceObject, viewType, invokers);
+            }
+        }
+    }
+
+    private static abstract class TypeSpecificMappingAction implements ViewDecoration, Serializable {
+        protected final Class<?> targetType;
+
+        TypeSpecificMappingAction(Class<?> targetType) {
+            this.targetType = targetType;
+        }
+
+        @Override
+        public boolean isNoOp() {
+            return false;
+        }
+
+        @Override
+        public ViewDecoration restrictTo(Set<Class<?>> viewTypes) {
+            if (viewTypes.contains(targetType)) {
+                return this;
+            }
+            return NO_OP_MAPPER;
+        }
+
+        @Override
+        public void collectInvokers(Object sourceObject, Class<?> viewType, List<MethodInvoker> invokers) {
+            if (targetType.isAssignableFrom(viewType)) {
+                invokers.add(createInvoker());
+            }
+        }
+
+        protected abstract MethodInvoker createInvoker();
+    }
+
+    private static class MixInBeanMappingAction extends TypeSpecificMappingAction {
+        private final Object mixIn;
+
+        MixInBeanMappingAction(Class<?> targetType, Object mixIn) {
+            super(targetType);
+            this.mixIn = mixIn;
+        }
+
+        @Override
+        public int hashCode() {
+            return targetType.hashCode() ^ mixIn.hashCode();
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            if (!obj.getClass().equals(MixInBeanMappingAction.class)) {
+                return false;
+            }
+            MixInBeanMappingAction other = (MixInBeanMappingAction) obj;
+            return targetType.equals(other.targetType) && mixIn.equals(other.mixIn);
+        }
+
+        @Override
+        protected MethodInvoker createInvoker() {
+            return new BeanMixInMethodInvoker(mixIn, REFLECTION_METHOD_INVOKER);
+        }
+    }
+
+    private static class MixInTypeMappingAction extends TypeSpecificMappingAction {
+        private final Class<?> mixInType;
+
+        MixInTypeMappingAction(Class<?> targetType, Class<?> mixInType) {
+            super(targetType);
+            this.mixInType = mixInType;
+        }
+
+        @Override
+        public int hashCode() {
+            return targetType.hashCode() ^ mixInType.hashCode();
+        }
+
+        @Override
+        public boolean equals(Object obj) {
+            if (!obj.getClass().equals(MixInTypeMappingAction.class)) {
+                return false;
+            }
+            MixInTypeMappingAction other = (MixInTypeMappingAction) obj;
+            return targetType.equals(other.targetType) && mixInType.equals(other.mixInType);
+        }
+
+        @Override
+        protected MethodInvoker createInvoker() {
+            return new ClassMixInMethodInvoker(mixInType, REFLECTION_METHOD_INVOKER);
+        }
+    }
+
+    private class DefaultViewBuilder<T> implements ViewBuilder<T> {
+        private final Class<T> viewType;
+        @Nullable
+        private final ViewGraphDetails graphDetails;
+        List<ViewDecoration> viewDecorations = new ArrayList<ViewDecoration>();
+
+        DefaultViewBuilder(Class<T> viewType) {
+            this.viewType = viewType;
+            this.graphDetails = null;
+        }
+
+        DefaultViewBuilder(Class<T> viewType, @Nullable ViewGraphDetails graphDetails) {
+            this.viewType = viewType;
+            this.graphDetails = graphDetails;
+        }
+
+        @Override
+        public ViewBuilder<T> mixInTo(final Class<?> targetType, final Object mixIn) {
+            viewDecorations.add(new MixInBeanMappingAction(targetType, mixIn));
+            return this;
+        }
+
+        @Override
+        public ViewBuilder<T> mixInTo(final Class<?> targetType, final Class<?> mixInType) {
+            viewDecorations.add(new MixInTypeMappingAction(targetType, mixInType));
+            return this;
+        }
+
+        @Override
+        public T build(@Nullable final Object sourceObject) {
+            if (sourceObject == null) {
+                return null;
+            }
+
+            ViewDecoration viewDecoration = MixInMappingAction.chain(viewDecorations);
+            return createView(viewType, sourceObject, viewDecoration, graphDetails != null ? graphDetails : new ViewGraphDetails(targetTypeProvider));
         }
     }
 }
