@@ -22,7 +22,19 @@ import java.io.File;
 import java.util.concurrent.TimeUnit;
 
 public interface GradleConnectionBuilderInternal extends GradleConnectionBuilder {
+
     GradleConnectionBuilderInternal daemonMaxIdleTime(int timeoutValue, TimeUnit timeoutUnits);
 
     GradleConnectionBuilderInternal daemonBaseDir(File daemonBaseDir);
+
+    GradleConnectionBuilderInternal integratedComposite(boolean integrated);
+
+    // The following only have an effect with `integratedComposite(true)
+    GradleConnectionBuilderInternal embedded(boolean embedded);
+
+    GradleConnectionBuilder useInstallation(File gradleHome);
+
+    GradleConnectionBuilder useGradleVersion(String version);
+
+    GradleConnectionBuilderInternal useClasspathDistribution();
 }
