@@ -64,7 +64,7 @@ public interface EclipseProject extends HierarchicalEclipseProject {
      * @return The dependencies. Returns an empty set if the project has no external dependencies.
      * @since 1.0-milestone-3
      */
-    DomainObjectSet<? extends ExternalDependency> getClasspath();
+    DomainObjectSet<? extends EclipseExternalDependency> getClasspath();
 
     /**
      * Returns the Eclipse natures configured on the project.
@@ -99,4 +99,24 @@ public interface EclipseProject extends HierarchicalEclipseProject {
      */
     @Incubating
     DomainObjectSet<? extends EclipseBuildCommand> getBuildCommands() throws UnsupportedMethodException;
+
+    /**
+     * Returns the Eclipse classpath containers defined on the project.
+     *
+     * @return The list of classpath containers.
+     * @since 3.0
+     * @throws UnsupportedMethodException For Gradle versions older than 3.0, where this method is not supported.
+     */
+    @Incubating
+    DomainObjectSet<? extends EclipseClasspathContainer> getClasspathContainers() throws UnsupportedMethodException;
+
+    /**
+     * Returns the output location of this project.
+     *
+     * @return The project's output location.
+     * @since 3.0
+     * @throws UnsupportedMethodException For Gradle versions older than 3.0, where this method is not supported.
+     */
+    @Incubating
+    EclipseOutputLocation getOutputLocation() throws UnsupportedMethodException;
 }
