@@ -32,9 +32,11 @@ import java.util.Set;
 import java.util.SortedSet;
 
 import org.grails.core.artefact.DomainClassArtefactHandler;
+import org.grails.datastore.mapping.model.config.GormProperties;
 import org.springframework.beans.BeanWrapper;
 import org.springframework.beans.BeanWrapperImpl;
 import org.springframework.context.MessageSource;
+import org.springframework.validation.BeanPropertyBindingResult;
 import org.springframework.validation.Errors;
 import org.springframework.validation.FieldError;
 
@@ -44,12 +46,14 @@ import org.springframework.validation.FieldError;
  *
  * @author Graeme Rocher
  * @since 0.1
+ * @deprecated Replaced by {@link grails.gorm.validation.PersistentEntityValidator}
  */
+@Deprecated
 public class GrailsDomainClassValidator implements CascadingValidator, GrailsApplicationAware {
 
     private static final List<String> EMBEDDED_EXCLUDES = Arrays.asList(
-        GrailsDomainClassProperty.IDENTITY,
-        GrailsDomainClassProperty.VERSION);
+        GormProperties.IDENTITY,
+        GormProperties.VERSION);
 
     protected Class<?> targetClass;
     protected GrailsDomainClass domainClass;
@@ -124,13 +128,19 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
      */
     protected void cascadeToAssociativeProperty(Errors errors, BeanWrapper bean, GrailsDomainClassProperty persistentProperty) {
         String propertyName = persistentProperty.getName();
-        if (errors.hasFieldErrors(propertyName)) return;
+        ConstrainedProperty c = (ConstrainedProperty) persistentProperty.getDomainClass().getConstrainedProperties().get(propertyName);
+        if(c != null) {
+            Object cascadeSetting = c.getMetaConstraintValue("cascade");
+            if (cascadeSetting == Boolean.FALSE) {
+                return;
+            }
+        }
 
         if (persistentProperty.isManyToOne() || persistentProperty.isOneToOne() || persistentProperty.isEmbedded()) {
             Object associatedObject = bean.getPropertyValue(propertyName);
             cascadeValidationToOne(errors, bean,associatedObject, persistentProperty, propertyName, null);
         }
-        else if (persistentProperty.isOneToMany()) {
+        else if (persistentProperty.isOneToMany() ) {
             cascadeValidationToMany(errors, bean, persistentProperty, propertyName);
         }
     }
@@ -147,17 +157,19 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
     @SuppressWarnings("rawtypes")
     protected void cascadeValidationToMany(Errors errors, BeanWrapper bean,
             GrailsDomainClassProperty persistentProperty, String propertyName) {
-
+        Object target = errors instanceof BeanPropertyBindingResult ? ((BeanPropertyBindingResult)errors).getTarget() : bean.getWrappedInstance();
         Object collection = bean.getPropertyValue(propertyName);
         if (collection instanceof List || collection instanceof SortedSet) {
             int idx = 0;
             for (Object associatedObject : ((Collection)collection)) {
+                if(associatedObject == target) continue;
                 cascadeValidationToOne(errors, bean,associatedObject, persistentProperty, propertyName, idx++);
             }
         }
         else if (collection instanceof Collection) {
             Integer index = 0;
             for (Object associatedObject : ((Collection)collection)) {
+                if(associatedObject == target) continue;
                 cascadeValidationToOne(errors, bean,associatedObject, persistentProperty, propertyName, index++);
             }
         }
@@ -166,7 +178,9 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
             filterGStringKeys((Map)collection);
             for (Object entryObject : ((Map) collection).entrySet()) {
                 Map.Entry entry = (Map.Entry) entryObject;
-                cascadeValidationToOne(errors, bean, entry.getValue(), persistentProperty, propertyName, entry.getKey());
+                Object associatedObject = entry.getValue();
+                if(associatedObject == target) continue;
+                cascadeValidationToOne(errors, bean, associatedObject, persistentProperty, propertyName, entry.getKey());
             }
         }
     }
@@ -174,7 +188,7 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
     @SuppressWarnings({ "rawtypes", "unchecked" })
     private void filterGStringKeys(Map collection) {
         Set set = collection.keySet();
-        Set<GString> gstrings = new HashSet<GString>();
+        Set<GString> gstrings = new HashSet<>();
         for (Object o : set) {
             if (o instanceof GString) {
                 gstrings.add((GString) o);
@@ -203,6 +217,10 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
         if (fieldError == null) {
             ConstrainedProperty c = (ConstrainedProperty) constrainedProperties.get(constrainedPropertyName);
             c.setMessageSource(messageSource);
+            Object cascadeSetting = c.getMetaConstraintValue("cascade");
+            if(cascadeSetting == Boolean.FALSE) {
+                return;
+            }
             c.validate(obj, bean.getPropertyValue(constrainedPropertyName), errors);
         }
     }
@@ -317,7 +335,7 @@ public class GrailsDomainClassValidator implements CascadingValidator, GrailsApp
      */
     public void setDomainClass(GrailsDomainClass domainClass) {
         this.domainClass = domainClass;
-        domainClass.setValidator(this);
+        //domainClass.setValidator(this);
         targetClass = domainClass.getClazz();
     }
 
