@@ -310,7 +310,7 @@ public class CharBuf extends Writer implements CharSequence {
             capacity = buffer.length;
         }
 
-        arraycopy(chars, 0, buffer, location, chars.length);
+        System.arraycopy(chars, 0, buffer, location, chars.length);
         location += chars.length;
         return this;
     }
@@ -328,7 +328,7 @@ public class CharBuf extends Writer implements CharSequence {
         _buffer[_location] = '"';
         _location++;
 
-        arraycopy(chars, 0, _buffer, _location, chars.length);
+        System.arraycopy(chars, 0, _buffer, _location, chars.length);
 
         _location += (chars.length);
         _buffer[_location] = '"';
@@ -386,7 +386,7 @@ public class CharBuf extends Writer implements CharSequence {
 
     final byte[] charTo = new byte[2];
 
-    private final CharBuf doAddJsonEscapedString(char[] charArray) {
+    private CharBuf doAddJsonEscapedString(char[] charArray) {
         char[] _buffer = buffer;
         int _location = this.location;
 
@@ -485,7 +485,8 @@ public class CharBuf extends Writer implements CharSequence {
                                 _location++;
                             }
                         } else {
-                            Byt.charTo(_charTo, c);
+                            _charTo[1] = (byte) (c);
+                            _charTo[0] = (byte) (c >>> 8);
 
                             for (int charByte : _charTo) {
                                 ByteScanner.encodeByteIntoTwoAsciiCharBytes(charByte, _encoded);
@@ -558,14 +559,6 @@ public class CharBuf extends Writer implements CharSequence {
         return this;
     }
 
-    private static final void sysstemarraycopy(final char[] src, final int srcPos, final char[] dest, final int destPos, final int length) {
-        System.arraycopy(src, srcPos, dest, destPos, length);
-    }
-
-    private static final void arraycopy(final char[] src, final int srcPos, final char[] dest, final int destPos, final int length) {
-        sysstemarraycopy(src, srcPos, dest, destPos, length);
-    }
-
     public CharBuf add(byte[] bytes, int start, int end) {
         int charsLength = end - start;
         if (charsLength + location > capacity) {
