@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2015 the original author or authors.
+ * Copyright 2012-2017 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -73,6 +73,7 @@ import org.springframework.util.Assert;
  * @author Andy Wilkinson
  * @author Phillip Webb
  * @author Andrey Stolyarov
+ * @author Henri Kerola
  */
 abstract class ArchiveCommand extends OptionParsingCommand {
 
@@ -104,6 +105,10 @@ abstract class ArchiveCommand extends OptionParsingCommand {
 			this.layout = layout;
 		}
 
+		protected Layout getLayout() {
+			return this.layout;
+		}
+
 		@Override
 		protected void doOptions() {
 			this.includeOption = option("include",
@@ -259,7 +264,7 @@ abstract class ArchiveCommand extends OptionParsingCommand {
 			if (classLoader == null) {
 				classLoader = Thread.currentThread().getContextClassLoader();
 			}
-			String name = sourceClass.replace(".", "/") + ".class";
+			String name = sourceClass.replace('.', '/') + ".class";
 			InputStream stream = classLoader.getResourceAsStream(name);
 			writer.writeEntry(this.layout.getClassesLocation() + name, stream);
 		}
@@ -278,13 +283,17 @@ abstract class ArchiveCommand extends OptionParsingCommand {
 					libraries.add(new Library(entry.getFile(), LibraryScope.COMPILE));
 				}
 				else {
-					writer.writeEntry(entry.getName(),
-							new FileInputStream(entry.getFile()));
+					writeClasspathEntry(writer, entry);
 				}
 			}
 			return libraries;
 		}
 
+		protected void writeClasspathEntry(JarWriter writer, MatchedResource entry)
+				throws IOException {
+			writer.writeEntry(entry.getName(), new FileInputStream(entry.getFile()));
+		}
+
 		protected abstract LibraryScope getLibraryScope(File file);
 
 	}
