@@ -27,6 +27,7 @@ import org.eclipse.jetty.server.NetworkConnector;
 import org.eclipse.jetty.server.Server;
 import org.eclipse.jetty.server.handler.HandlerCollection;
 import org.eclipse.jetty.server.handler.HandlerWrapper;
+import org.eclipse.jetty.util.component.AbstractLifeCycle;
 
 import org.springframework.boot.context.embedded.EmbeddedServletContainer;
 import org.springframework.boot.context.embedded.EmbeddedServletContainerException;
@@ -84,11 +85,21 @@ public class JettyEmbeddedServletContainer implements EmbeddedServletContainer {
 	private void initialize() {
 		synchronized (this.monitor) {
 			try {
-				// Cache and clear the connectors to prevent requests being handled before
-				// the application context is ready
+				// Cache the connectors and then remove them to prevent requests being
+				// handled before the application context is ready.
 				this.connectors = this.server.getConnectors();
-				this.server.setConnectors(null);
+				this.server.addBean(new AbstractLifeCycle() {
 
+					@Override
+					protected void doStart() throws Exception {
+						for (Connector connector : JettyEmbeddedServletContainer.this.connectors) {
+							Assert.state(connector.isStopped(), "Connector " + connector
+									+ " has been started prematurely");
+						}
+						JettyEmbeddedServletContainer.this.server.setConnectors(null);
+					}
+
+				});
 				// Start the server so that the ServletContext is available
 				this.server.start();
 				this.server.setStopAtShutdown(false);
