@@ -26,6 +26,7 @@ import org.springframework.context.ApplicationListener;
 import org.springframework.context.ConfigurableApplicationContext;
 import org.springframework.context.event.ApplicationEventMulticaster;
 import org.springframework.context.event.SimpleApplicationEventMulticaster;
+import org.springframework.context.support.AbstractApplicationContext;
 import org.springframework.core.Ordered;
 import org.springframework.core.env.ConfigurableEnvironment;
 import org.springframework.util.ErrorHandler;
@@ -62,7 +63,8 @@ public class EventPublishingRunListener implements SpringApplicationRunListener,
 	}
 
 	@Override
-	public void started() {
+	@SuppressWarnings("deprecation")
+	public void starting() {
 		this.initialMulticaster
 				.multicastEvent(new ApplicationStartedEvent(this.application, this.args));
 	}
@@ -93,12 +95,20 @@ public class EventPublishingRunListener implements SpringApplicationRunListener,
 	@Override
 	public void finished(ConfigurableApplicationContext context, Throwable exception) {
 		SpringApplicationEvent event = getFinishedEvent(context, exception);
-		if (context != null) {
+		if (context != null && context.isActive()) {
 			// Listeners have been registered to the application context so we should
 			// use it at this point if we can
 			context.publishEvent(event);
 		}
 		else {
+			// An inactive context may not have a multicaster so we use our multicaster to
+			// call all of the context's listeners instead
+			if (context instanceof AbstractApplicationContext) {
+				for (ApplicationListener<?> listener : ((AbstractApplicationContext) context)
+						.getApplicationListeners()) {
+					this.initialMulticaster.addApplicationListener(listener);
+				}
+			}
 			if (event instanceof ApplicationFailedEvent) {
 				this.initialMulticaster.setErrorHandler(new LoggingErrorHandler());
 			}
