@@ -24,6 +24,8 @@ import groovy.lang.MetaClass;
 import groovy.lang.MetaClassRegistry;
 import groovy.lang.MetaProperty;
 import groovy.util.ConfigObject;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.springframework.beans.BeanInstantiationException;
 import org.springframework.beans.BeanUtils;
 import org.springframework.beans.BeanWrapper;
@@ -60,6 +62,7 @@ import java.util.TreeSet;
  */
 public class GrailsClassUtils {
 
+    private static final Log LOG = LogFactory.getLog(GrailsClassUtils.class);
     public static final Map<Class<?>, Class<?>> PRIMITIVE_TYPE_COMPATIBLE_CLASSES = new HashMap<Class<?>, Class<?>>();
 
     /**
@@ -322,8 +325,11 @@ public class GrailsClassUtils {
         }
 
         Set<PropertyDescriptor> properties = new HashSet<PropertyDescriptor>();
+        PropertyDescriptor descriptor = null;
         try {
-            for (PropertyDescriptor descriptor : BeanUtils.getPropertyDescriptors(clazz)) {
+            PropertyDescriptor[] descriptors = BeanUtils.getPropertyDescriptors(clazz);
+            for (int i = 0; i < descriptors.length; i++) {
+                descriptor = descriptors[i];
                 Class<?> currentPropertyType = descriptor.getPropertyType();
                 if (isTypeInstanceOfPropertyType(propertyType, currentPropertyType)) {
                     properties.add(descriptor);
@@ -331,6 +337,11 @@ public class GrailsClassUtils {
             }
         }
         catch (Exception e) {
+            if(descriptor == null) {
+                LOG.error(String.format("Got exception while checking property descriptors for class %s", clazz.getName()), e);
+            } else {
+                LOG.error(String.format("Got exception while checking PropertyDescriptor.propertyType for field %s.%s", clazz.getName(), descriptor.getName()), e);
+            }
             // if there are any errors in instantiating just return null for the moment
             return new PropertyDescriptor[0];
         }
@@ -352,14 +363,23 @@ public class GrailsClassUtils {
         if (clazz == null || propertySuperType == null) return new PropertyDescriptor[0];
 
         Set<PropertyDescriptor> properties = new HashSet<PropertyDescriptor>();
+        PropertyDescriptor descriptor = null;
         try {
-            for (PropertyDescriptor descriptor : BeanUtils.getPropertyDescriptors(clazz)) {
+            PropertyDescriptor[] descriptors = BeanUtils.getPropertyDescriptors(clazz);
+            for (int i = 0; i < descriptors.length; i++) {
+                descriptor = descriptors[i];
+                Class<?> currentPropertyType = descriptor.getPropertyType();
                 if (propertySuperType.isAssignableFrom(descriptor.getPropertyType())) {
                     properties.add(descriptor);
                 }
             }
         }
         catch (Exception e) {
+            if(descriptor == null) {
+                LOG.error(String.format("Got exception while checking property descriptors for class %s", clazz.getName()), e);
+            } else {
+                LOG.error(String.format("Got exception while checking PropertyDescriptor.propertyType for field %s.%s", clazz.getName(), descriptor.getName()), e);
+            }
             return new PropertyDescriptor[0];
         }
         return properties.toArray(new PropertyDescriptor[properties.size()]);
@@ -689,7 +709,7 @@ public class GrailsClassUtils {
      * @return true if the method is a property getter
      */
     public static boolean isPropertyGetter(Method method) {
-        return !Modifier.isStatic(method.getModifiers()) && Modifier.isPublic(method.getModifiers()) && isGetter(method.getName(), method.getReturnType(), method.getParameterTypes());
+        return !Modifier.isStatic(method.getModifiers()) && Modifier.isPublic(method.getModifiers()) && GrailsNameUtils.isGetter(method.getName(), method.getReturnType(), method.getParameterTypes());
     }
 
     /**
@@ -712,180 +732,6 @@ public class GrailsClassUtils {
         return elements;
     }
 
-    /**
-     * Retrieves the name of a setter for the specified property name
-     * @param propertyName The property name
-     * @return The setter equivalent
-     */
-    public static String getSetterName(String propertyName) {
-        return GrailsNameUtils.getSetterName(propertyName);
-    }
-
-    /**
-     * Returns true if the name of the method specified and the number of arguments make it a javabean property getter.
-     * The name is assumed to be a valid Java method name, that is not verified.
-     *
-     * @param name The name of the method
-     * @param args The arguments
-     * @return true if it is a javabean property getter
-     * @deprecated use {@link #isGetter(String, Class, Class[])} instead because this method has a defect for "is.." method with Boolean return types.
-     */
-    public static boolean isGetter(String name, Class<?>[] args) {
-        return isGetter(name, boolean.class, args);
-    }
-
-    /**
-     * Returns true if the name of the method specified and the number of arguments make it a javabean property getter.
-     * The name is assumed to be a valid Java method name, that is not verified.
-     *
-     * @param name The name of the method
-     * @param returnType The return type of the method
-     * @param args The arguments
-     * @return true if it is a javabean property getter
-     */
-    public static boolean isGetter(String name, Class returnType, Class<?>[] args) {
-        if (!StringUtils.hasText(name) || args == null)return false;
-        if (args.length != 0)return false;
-
-        if (name.startsWith("get")) {
-            name = name.substring(3);
-            if (isPropertyMethodSuffix(name)) return true;
-        }
-        else if (name.startsWith("is") && returnType == boolean.class) {
-            name = name.substring(2);
-            if (isPropertyMethodSuffix(name)) return true;
-        }
-        return false;
-    }
-
-    /**
-     * This method is used when interrogating a method name to determine if the
-     * method represents a property getter.  For example, if a method is named
-     * <code>getSomeProperty</code>, the value <code>"SomeProperty"</code> could
-     * be passed to this method to determine that the method should be considered
-     * a property getter.  Examples of suffixes that would be considered property
-     * getters:
-     * <ul>
-     *     <li>SomeProperty</li>
-     *     <li>Word</li>
-     *     <li>aProperty</li>
-     *     <li>S</li>
-     *     <li>X567</li>
-     * </ul>
-     *
-     * Examples of suffixes that would not be considered property getters:
-     * <ul>
-     *     <li>someProperty</li>
-     *     <li>word</li>
-     *     <li>s</li>
-     *     <li>x567</li>
-     *     <li>2other</li>
-     *     <li>5</li>
-     * </ul>
-     *
-     * A suffix like <code>prop</code> from a method <code>getprop()</code> is
-     * not recognized as a valid suffix. However Groovy will recognize such a
-     * method as a property getter but only if a method <code>getProp()</code> or
-     * a property <code>prop</code> does not also exist. The Java Beans
-     * specification is unclear on how to treat such method names, it only says
-     * that "by default" the suffix will start with a capital letter because of
-     * the camel case style usually used. (See the JavaBeans API specification
-     * sections 8.3 and 8.8.)
-     *
-     * This method assumes that all characters in the name are valid Java identifier
-     * letters.
-     *
-     * @param suffix The suffix to inspect
-     * @return true if suffix indicates a property name
-     */
-    protected static boolean isPropertyMethodSuffix(String suffix) {
-        if(suffix.length() == 0) return false;
-        if(!Character.isJavaIdentifierStart(suffix.charAt(0))) return false;
-        if(suffix.length() == 1) return Character.isUpperCase(suffix.charAt(0));
-        return Character.isUpperCase(suffix.charAt(0)) || Character.isUpperCase(suffix.charAt(1));
-    }
-
-    /**
-     * Returns a property name equivalent for the given getter name or null if it is not a valid getter. If not null
-     * or empty the getter name is assumed to be a valid identifier.
-     *
-     * @param getterName The getter name
-     * @return The property name equivalent
-     * @deprecated Use {@link #getPropertyForGetter(String, Class)} instead because this method has a defect for "is.." method with Boolean return types.
-     */
-    public static String getPropertyForGetter(String getterName) {
-        return getPropertyForGetter(getterName, boolean.class);
-    }
-
-    /**
-     * Returns a property name equivalent for the given getter name and return type or null if it is not a valid getter. If not null
-     * or empty the getter name is assumed to be a valid identifier.
-     *
-     * @param getterName The getter name
-     * @param returnType The type the method returns
-     * @return The property name equivalent
-     */
-    public static String getPropertyForGetter(String getterName, Class returnType) {
-        if (getterName == null || getterName.length() == 0) return null;
-
-        if (getterName.startsWith("get")) {
-            String prop = getterName.substring(3);
-            return convertValidPropertyMethodSuffix(prop);
-        }
-        if (getterName.startsWith("is") && returnType == boolean.class) {
-            String prop = getterName.substring(2);
-            return convertValidPropertyMethodSuffix(prop);
-        }
-        return null;
-    }
-
-    /**
-     * This method functions the same as {@link #isPropertyMethodSuffix(String)},
-     * but in addition returns the property name, or null if not a valid property.
-     *
-     * @param suffix The suffix to inspect
-     * @return The property name or null
-     */
-    private static String convertValidPropertyMethodSuffix(String suffix) {
-        if (suffix.length() == 0) return null;
-
-        // We assume all characters are Character.isJavaIdentifierPart, but the first one may not be a valid
-        // starting character.
-        if (!Character.isJavaIdentifierStart(suffix.charAt(0))) return null;
-
-        if (suffix.length() == 1) {
-            return Character.isUpperCase(suffix.charAt(0)) ? suffix.toLowerCase() : null;
-        }
-        if (Character.isUpperCase(suffix.charAt(1))) {
-            // "aProperty", "AProperty"
-            return suffix;
-        }
-        if (Character.isUpperCase(suffix.charAt(0))) {
-            return Character.toLowerCase(suffix.charAt(0)) + suffix.substring(1);
-        }
-        if('_' == suffix.charAt(0)) {
-            return suffix;
-        }
-        return null;
-    }
-
-    /**
-     * Returns a property name equivalent for the given setter name or null if it is not a valid setter. If not null
-     * or empty the setter name is assumed to be a valid identifier.
-     *
-     * @param setterName The setter name, must be null or empty or a valid identifier name
-     * @return The property name equivalent
-     */
-    public static String getPropertyForSetter(String setterName) {
-        if (setterName == null || setterName.length() == 0) return null;
-
-        if (setterName.startsWith("set")) {
-            String prop = setterName.substring(3);
-            return convertValidPropertyMethodSuffix(prop);
-        }
-        return null;
-    }
-
     /**
      * Returns true if the name of the method specified and the number of arguments make it a javabean property setter.
      * The name is assumed to be a valid Java method name, that is not verified.
@@ -900,7 +746,7 @@ public class GrailsClassUtils {
 
         if (name.startsWith("set")) {
             if (args.length != 1) return false;
-            return isPropertyMethodSuffix(name.substring(3));
+            return GrailsNameUtils.isPropertyMethodSuffix(name.substring(3));
         }
 
         return false;
@@ -1013,6 +859,77 @@ public class GrailsClassUtils {
         return null;
     }
 
+    /**
+     * Retrieves the name of a setter for the specified property name
+     * @param propertyName The property name
+     * @return The setter equivalent
+     */
+    public static String getSetterName(String propertyName) {
+        return GrailsNameUtils.getSetterName(propertyName);
+    }
+
+    /**
+     * Returns true if the name of the method specified and the number of arguments make it a javabean property getter.
+     * The name is assumed to be a valid Java method name, that is not verified.
+     *
+     * @param name The name of the method
+     * @param args The arguments
+     * @return true if it is a javabean property getter
+     * @deprecated use {@link #isGetter(String, Class, Class[])} instead because this method has a defect for "is.." method with Boolean return types.
+     */
+    public static boolean isGetter(String name, Class<?>[] args) {
+        return GrailsNameUtils.isGetter(name, boolean.class, args);
+    }
+
+    /**
+     * Returns true if the name of the method specified and the number of arguments make it a javabean property getter.
+     * The name is assumed to be a valid Java method name, that is not verified.
+     *
+     * @param name The name of the method
+     * @param returnType The return type of the method
+     * @param args The arguments
+     * @return true if it is a javabean property getter
+     */
+    public static boolean isGetter(String name, Class returnType, Class<?>[] args) {
+        return GrailsNameUtils.isGetter(name, returnType, args);
+    }
+
+    /**
+     * Returns a property name equivalent for the given getter name or null if it is not a valid getter. If not null
+     * or empty the getter name is assumed to be a valid identifier.
+     *
+     * @param getterName The getter name
+     * @return The property name equivalent
+     * @deprecated Use {@link #getPropertyForGetter(String, Class)} instead because this method has a defect for "is.." method with Boolean return types.
+     */
+    public static String getPropertyForGetter(String getterName) {
+        return GrailsNameUtils.getPropertyForGetter(getterName);
+    }
+
+    /**
+     * Returns a property name equivalent for the given getter name and return type or null if it is not a valid getter. If not null
+     * or empty the getter name is assumed to be a valid identifier.
+     *
+     * @param getterName The getter name
+     * @param returnType The type the method returns
+     * @return The property name equivalent
+     */
+    public static String getPropertyForGetter(String getterName, Class returnType) {
+        return GrailsNameUtils.getPropertyForGetter(getterName, returnType);
+    }
+
+
+    /**
+     * Returns a property name equivalent for the given setter name or null if it is not a valid setter. If not null
+     * or empty the setter name is assumed to be a valid identifier.
+     *
+     * @param setterName The setter name, must be null or empty or a valid identifier name
+     * @return The property name equivalent
+     */
+    public static String getPropertyForSetter(String setterName) {
+        return GrailsNameUtils.getPropertyForSetter(setterName);
+    }
+
     /**
      * Returns whether the specified class is either within one of the specified packages or
      * within a subpackage of one of the packages
